-- Creates latest IRM database
--
-- *** WARNING: drops all IRM tables if they exist *** 
--

SET SCHEMA 'public';


-- JOBFLOW_INSTANCE
--------------------------------------------------------------------------------------------------------
DROP TABLE IF EXISTS JOBFLOW_INSTANCE;

CREATE TABLE JOBFLOW_INSTANCE
(
   INSTANCE_KEY bigint NOT NULL,
   INSTANCE_NAME character varying(60) NOT NULL,
   INSTANCE_DESC character varying(255),
   
   DEFINITION_FILE character varying(80) NOT NULL,
   
   ENTITY_ID character varying(32) NOT NULL,   
   ENTITY_ROLE_ID character varying(32) NOT NULL,
   
   CATEGORY_ID character varying(80),
   CONFIG_SET_ID character varying(32) NOT NULL,
   
   BASE_DATE character varying(32) NOT NULL,
   
   CREATED_BY character varying(32),
   CREATED_BY_ID character varying(32),
   CREATED_DTTM TIMESTAMP,
   LAST_MODIFIED_BY character varying(32),

   STATUS_CD integer NOT NULL,
   
   PROCESS_KEY bigint NOT NULL,
   ROOT_PROCESS_KEY bigint NOT NULL,
   AUTO_DELETE_FLG boolean,
   STARTUP_FLG boolean,
   INPUT_DATA_MODIFIED_FLG boolean,
   DEBUG_FLG boolean,
   VISIBILITY_CD integer NOT NULL,   
   
   FEDERATED_AREA_ID character varying(32) NOT NULL,
   
   LAST_RUN_DTTM TIMESTAMP,
   
   PUBLISHED_BY character varying(32),
   PUBLISHED_DTTM TIMESTAMP,
   PUBLISHED_COMMENTS character varying(255),
   
   DEFAULT_INSTANCE_FLG boolean,
   
   CONSTRAINT PK_JOBFLOW_INSTANCE PRIMARY KEY (INSTANCE_KEY)
);

-- JOBFLOW_TASK
--------------------------------------------------------------------------------------------------------
DROP TABLE IF EXISTS JOBFLOW_TASK;

CREATE TABLE JOBFLOW_TASK
(
   TASK_KEY bigint NOT NULL,
   TASK_NAME character varying(60) NOT NULL,
   TASK_ID character varying(32) NOT NULL, 
   TASK_TYPE_CD integer NOT NULL,
   REMOTE_FLG boolean,
   FEDERATED_AREA_ID character varying(32),
   PROCESS_KEY bigint NOT NULL,
   ROOT_PROCESS_KEY bigint NOT NULL,
   FLOW_ORDER integer NOT NULL,
   STATUS_CD integer NOT NULL,
   CONSTRAINT PK_JAVA_TASK PRIMARY KEY (TASK_KEY, PROCESS_KEY)
);

ALTER TABLE JOBFLOW_TASK ADD CONSTRAINT UNIQUE_TASK_ID UNIQUE (TASK_ID, PROCESS_KEY);

-- NODE_DATA
--------------------------------------------------------------------------------------------------------
DROP TABLE IF EXISTS NODE_DATA;

CREATE TABLE NODE_DATA
(
   DATA_KEY bigint NOT NULL,
   DATA_NAME character varying(80) NOT NULL,
   DATA_ID character varying(32) NOT NULL, 
   DATA_TYPE character varying(16) NOT NULL,
   
   COLLECTION_FLG boolean,
   PARTITION_FLG boolean,
   MODIFIED_FLG boolean,
   READONLY_FLG boolean,   
   RESULT_FLG boolean,
   
   URL character varying(2083),
   PATH character varying(4096),
   RAW_PATH character varying(4096),   
      
   NODE_KEY bigint NOT NULL,
   PROCESS_KEY bigint NOT NULL,
   
   DATA_STATE character varying(80),
   MACRO_VAR character varying(32), 
   SOURCE_ID character varying(32), 
   
   TIME_STAMP bigint NOT NULL,
   
   BASE_NAME character varying(60),
   LIBNAME character varying(32),
   EXT character varying(8),   
   FEDERATED_AREA_ID character varying(32),
   
   MODIFIED_BY character varying(32),
   MODIFIED_DTTM TIMESTAMP,
   MODIFIED_COMMENTS character varying(255),
      
   CONSTRAINT PK_NODE_DATA PRIMARY KEY (DATA_KEY)
);

ALTER TABLE NODE_DATA ADD CONSTRAINT UNIQUE_DATA_ID UNIQUE (DATA_ID, PROCESS_KEY);

-- DATA_OBJ
--------------------------------------------------------------------------------------------------------
DROP TABLE IF EXISTS DATA_OBJ;

CREATE TABLE DATA_OBJ
(
   DATA_OBJ_KEY SERIAL PRIMARY KEY,
   DATA_OBJ_NAME character varying(80) NOT NULL,
   DATA_OBJ_ID character varying(80) NOT NULL,   
   
   ROOT_CATEGORY_ID character varying(80),
   ROOT_DEFINITION_FILE character varying(80) NOT NULL,     
   ROOT_FEDERATED_AREA_ID character varying(32) NOT NULL,   
      
   CATEGORY_ID character varying(80),
   DEFINITION_FILE character varying(80) NOT NULL,   
   DEFINITION_FILE_LVL integer NOT NULL,
   PROCESS_ID character varying(80) NOT NULL,   
   
   READONLY_FLG boolean,   
      
   URL character varying(2083),
   PATH character varying(4096),
   RAW_PATH character varying(4096),   
            
   BASE_NAME character varying(60),
   LIBNAME character varying(32),
   EXT character varying(8),   
   LIBREF_FLG boolean DEFAULT FALSE,
   FEDERATED_AREA_ID character varying(32),
   
   MV_NAMES character varying(255),
   MV_VALUES character varying(255)           
);


-- DATA_OBJ_REF 
--------------------------------------------------------------------------------------------------------
DROP TABLE IF EXISTS DATA_OBJ_REF;

CREATE TABLE DATA_OBJ_REF
(
   DATA_OBJ_REF_KEY SERIAL PRIMARY KEY,
   DATA_OBJ_REF_ID character varying(32) NOT NULL, 
   
   -- input=1; output=2
   DATA_OBJ_REF_TYPE integer NOT NULL, 
   
   DATA_OBJ_KEY bigint NOT NULL,
   
   DATA_TYPE character varying(16) NOT NULL,
         
   PARENT_ID character varying(32) NOT NULL, 

   COLLECTION_FLG boolean,
   
   DATA_STATE character varying(80),   
   MACRO_VAR character varying(32), 
   PARTITION_FLG boolean,
   
   SOURCE_ID character varying(32),
   
   RESULT_FLG boolean, 
   MODIFIED_FLG boolean   
);


-- DATA_OBJ_OVRD
--------------------------------------------------------------------------------------------------------
DROP TABLE IF EXISTS DATA_OBJ_OVRD;

CREATE TABLE DATA_OBJ_OVRD
(
   DATA_OBJ_OVRD_KEY SERIAL PRIMARY KEY,
   
   DATA_OBJ_KEY bigint NOT NULL,   
   INSTANCE_KEY bigint NOT NULL,
      
   OVRD_URL character varying(2083),
   OVRD_PATH character varying(4096),
   OVRD_RAW_PATH character varying(4096),   
               
   MODIFIED_BY character varying(32),
   MODIFIED_DTTM TIMESTAMP,
   MODIFIED_COMMENTS character varying(255)     
);


-- DATA_OBJ_REF_SET
--------------------------------------------------------------------------------------------------------
DROP TABLE IF EXISTS DATA_OBJ_REF_SET;

CREATE TABLE DATA_OBJ_REF_SET
(
   DATA_OBJ_REF_SET_KEY bigint NOT NULL,
   DATA_OBJ_REF_KEY  bigint NOT NULL,
   
   PRIMARY KEY(DATA_OBJ_REF_SET_KEY, DATA_OBJ_REF_KEY)
);


-- INST_DATA_OBJ_REF_SET
--------------------------------------------------------------------------------------------------------
DROP TABLE IF EXISTS INST_DATA_OBJ_REF_SET;

CREATE TABLE INST_DATA_OBJ_REF_SET
(
   ROW_KEY SERIAL PRIMARY KEY,
   DATA_OBJ_REF_SET_KEY bigint NOT NULL,
   INSTANCE_KEY bigint NOT NULL
);

ALTER TABLE INST_DATA_OBJ_REF_SET ADD CONSTRAINT UNIQUE_DATA_INST_REF_SET UNIQUE (DATA_OBJ_REF_SET_KEY, INSTANCE_KEY);




-- legacy table --
DROP TABLE IF EXISTS APP_USER;
CREATE TABLE APP_USER
(
        APP_USER_SK                        INTEGER NOT NULL,
        APP_USER_ID                        VARCHAR(40) NOT NULL,
        APP_USER_NM                        VARCHAR(60),
        APP_USER_ROLE                      VARCHAR(40),
        LAST_LOGIN_DTTM                    TIMESTAMP,
        LAST_AUDIT_TRAIL_READ_DTTM         TIMESTAMP,
        PRIMARY KEY (APP_USER_SK)
);

CREATE UNIQUE INDEX APP_USER_IX1 ON APP_USER (APP_USER_ID);
DROP SEQUENCE IF EXISTS SEQ_APP_USER_SK;
CREATE SEQUENCE SEQ_APP_USER_SK START WITH 1;


-- Data Object Pooling Tables --
-- The inputs of tasks.
DROP TABLE IF EXISTS INPUT_POOL;
CREATE TABLE INPUT_POOL
(
   INPUT_KEY      bigint NOT NULL,
   INPUT_NAME     character varying(255) NOT NULL,      -- The 3-part name
   FEDERATEDID    character varying(255) NOT NULL,      -- The federated area id
   DATA_OBJECTID  character varying(255) NOT NULL,      -- The federated area id + 3-part name
   PATH           character varying(4096),     		-- The pathname on the Workspace Server machine. Should only be used for files in LA (null for all others).
   POOLED_FOLDER  character varying(4096),     		-- The pathname to the pooled folder on the Workspace Server machine.
   TIME_STAMP     bigint NOT NULL,			-- The last modification time stamp
   CHECKSUM       character varying(32),		-- The checksum of the data inside the file (used to speed up DataPoolingService.fileUploaded). NULL for tables in LA. 
         
   CONSTRAINT PK_INPUT_POOL PRIMARY KEY (INPUT_KEY)
);

DROP SEQUENCE IF EXISTS INPUT_POOL_SEQUENCE;
CREATE SEQUENCE INPUT_POOL_SEQUENCE START 101;

-- The outputs of tasks.
DROP TABLE IF EXISTS OUTPUT_POOL;
CREATE TABLE OUTPUT_POOL
(
   OUTPUT_KEY         bigint NOT NULL,
   TASKDEFID          character varying(255) NOT NULL,        -- The FEDERATEDID::TASK-IMPLEMENTATION[substitution-list] that uniquely identifies a task
   GRAPH_PATH_KEY     bigint NOT NULL,
   POOLED_FOLDER      character varying(4096),                -- The folder for the output of the task (is NULL when output is not available)   
   OUTPUT_AVAILABLE   boolean NOT NULL,			              -- The file length (used to speed up DataPoolingService.fileUploaded)
   OUT_OF_DATE   	  boolean NOT NULL,			              -- the output is out-of-date due to input data from FAs changed
   OUTPUT_DATANAMES   character varying not null default '',  -- the sorted output data names of the corresponding task
         
   CONSTRAINT PK_OUTPUT_POOL PRIMARY KEY (OUTPUT_KEY)
);

DROP SEQUENCE IF EXISTS OUTPUT_POOL_SEQUENCE;
CREATE SEQUENCE OUTPUT_POOL_SEQUENCE START 101;

-- A path in the graph 
DROP TABLE IF EXISTS GRAPH_PATH;
CREATE TABLE GRAPH_PATH
(
   GRAPH_PATH_KEY     bigint NOT NULL,
   TASKDEFID_LIST     character varying NOT NULL,           -- The list of FEDERATEDID::TASK-IMPLEMENTATION 

   CONSTRAINT PK_GRAPH_PATH PRIMARY KEY (GRAPH_PATH_KEY)
);

DROP SEQUENCE IF EXISTS GRAPH_PATH_SEQUENCE;
CREATE SEQUENCE GRAPH_PATH_SEQUENCE START 101;

-- An association table between graph paths and inputs
DROP TABLE IF EXISTS GRAPH_PATH_INPUT;
CREATE TABLE GRAPH_PATH_INPUT
(
   GRAPH_PATH_KEY     bigint NOT NULL,
   INPUT_KEY          bigint NOT NULL, 
         
   CONSTRAINT PK_GRAPH_PATH_INPUT PRIMARY KEY (GRAPH_PATH_KEY, INPUT_KEY)
);

DROP SEQUENCE IF EXISTS GRAPH_PATH_SEQUENCE;
CREATE SEQUENCE GRAPH_PATH_SEQUENCE START 101;

-- An association table between job flow instance keys and their tasks along with the outputs of each task
DROP TABLE IF EXISTS INSTANCE_TASK_OUTPUT;
CREATE TABLE INSTANCE_TASK_OUTPUT
(
   INSTANCE_KEY bigint NOT NULL,	  -- The jobflow instance key 
   TASK_KEY     bigint NOT NULL,      -- The JOB FLOW TASK KEY
   OUTPUT_KEY   bigint NOT NULL,
         
   CONSTRAINT PK_INSTANCE_TASK_OUTPUT PRIMARY KEY (INSTANCE_KEY, TASK_KEY, OUTPUT_KEY)
);

-- A table containing existing unique task data pathes (input and output) and associated keys 
DROP TABLE IF EXISTS DATA_PATH;
CREATE TABLE DATA_PATH
(
   DATA_PATH_KEY     bigint NOT NULL,
   DATA_PATH         character varying(4096) NOT NULL,
         
   CONSTRAINT PK_DATA_PATH PRIMARY KEY (DATA_PATH_KEY),
   CONSTRAINT IX_DATA_PATH UNIQUE (DATA_PATH)
);
DROP SEQUENCE IF EXISTS DATA_PATH_SEQUENCE;
CREATE SEQUENCE DATA_PATH_SEQUENCE START 101;

-- An association table between job flow instance keys and their tasks along with the minimum inputs of each task
DROP TABLE IF EXISTS INSTANCE_INPUT;
CREATE TABLE INSTANCE_INPUT
(
   INSTANCE_KEY bigint NOT NULL,	  -- The jobflow instance key 
   INPUT_KEY    bigint NOT NULL,
         
   CONSTRAINT PK_INSTANCE_INPUT PRIMARY KEY (INSTANCE_KEY, INPUT_KEY)
);

-- SYSTEM_OPTION_MAP
--------------------------------------------------------------------------------------------------------
DROP TABLE IF EXISTS SYSTEM_OPTION_MAP;

CREATE TABLE SYSTEM_OPTION_MAP
(   
   OBJ_KEY SERIAL PRIMARY KEY,
   BASE_DATE character varying(32) NOT NULL,  
   CONFIG_SET_ID character varying(80) NOT NULL,
   ENTITY_ID character varying(80) NOT NULL,
   GROUP_FLG character varying(8) NOT NULL,
   PATH character varying(4096) NOT NULL   
);

ALTER TABLE SYSTEM_OPTION_MAP ADD CONSTRAINT UNIQUE_DATA_SYSTEM_OPTION_MAP UNIQUE (BASE_DATE, CONFIG_SET_ID, ENTITY_ID, GROUP_FLG);

-- CAS Session Tracking

DROP SEQUENCE IF EXISTS SEQ_CAS_SESSION_KEY;

CREATE SEQUENCE SEQ_CAS_SESSION_KEY START WITH 1;

DROP TABLE IF EXISTS CAS_SESSION;

CREATE TABLE CAS_SESSION (
  CAS_SESSION_KEY INTEGER NOT NULL,
  NAME            VARCHAR(100) NOT NULL,
  UUID            VARCHAR(100),
  STATE           VARCHAR(255),
  AUTH            VARCHAR(255),
  USERID          VARCHAR(100),
  CREATED         TIMESTAMP NOT NULL,
  INUSE           INTEGER NOT NULL DEFAULT 0,
  PRIMARY KEY(CAS_SESSION_KEY)
) ;

DROP INDEX IF EXISTS CAS_NAME_IDX;

CREATE UNIQUE INDEX CAS_NAME_IDX ON CAS_SESSION (NAME);

-- CAS Library Tracking

DROP SEQUENCE IF EXISTS SEQ_CAS_LIB_KEY;

CREATE SEQUENCE SEQ_CAS_LIB_KEY START WITH 1;

DROP TABLE IF EXISTS CAS_LIB;

CREATE TABLE CAS_LIB (
  CAS_LIB_KEY INTEGER NOT NULL,
  NAME        VARCHAR(256) NOT NULL,
  POOL_ID     VARCHAR(100) NOT NULL,
  DATASOURCE  VARCHAR(255),
  PATH        VARCHAR(255),
  FA_ID       VARCHAR(100),
  PRIMARY KEY(CAS_LIB_KEY)
) ;

DROP INDEX IF EXISTS CASLIB_NAME_IDX;

CREATE UNIQUE INDEX CASLIB_NAME_IDX ON CAS_LIB (NAME);

DROP INDEX IF EXISTS CASLIB_ID_IDX;

CREATE UNIQUE INDEX CASLIB_ID_IDX ON CAS_LIB (POOL_ID);

DROP TABLE IF EXISTS TASK_INFO;

CREATE TABLE TASK_INFO
(
   FEDERATED_AREA_ID character varying(32) PRIMARY KEY,
   TASK_JSON text NOT NULL,
   LAST_MODIFIED_TIMESTAMP bigint NOT NULL
);
