/*
 Copyright (C) 2017 SAS Institute Inc. Cary, NC, USA
*/

/**
   \file
   \brief The macro retrieves entities' own companies and their immediate child entities.
			IN solo case, this macros only retrive non-legal entities where SOLO_PARENT_FLG is set to Y.
       
   <b> Identified Inputs </b>

   \param[in]  IN_ENTITY_DS                   input table with entity information

   \param[in]  IN_ENTITY_ROLE                 input child entity role SOLO or GROUP
   
   \param[in]  CHILD_LIST                     input child entity list
                                              
   <b> Identified Outputs</b>

   \param[out] O_ENTITY_FINAL_DS              output table with solo entity and its non-legal entities

   \param[out] O_ENTITY_SUB_GRP_DS            output table for entities that have child entities
   
   
 
   The table O_ENTITY_FINAL_DS has the following structure:
      
      |          Variable              | Type     | Expected contents               |
      |--------------------------------|----------|---------------------------------|
      | ENTITY_ID                      | Char(32) | Entity Identification           |
      | CURRENCY_CD                    | Char(32) | Currency code                   |
      | GROUP_ID                       | Char(32) | Group Identification            |
      | ENTITY_ROLE_CD                 | Char(5)  | Entity role code                |


   
    \ingroup CommonAnalytics
   
    \author SAS Institute INC.
    \date 2017
    
 */

%macro irm_get_child_parent_entity(	IN_ENTITY_DS=,
                                	CHILD_LIST=,
                                	IN_ENTITY_ROLE=GROUP,
                                	O_ENTITY_FINAL_DS=final_entities,
                                	O_ENTITY_SUB_GRP_DS=sub_groups);
  
  		%if &IN_ENTITY_DS eq %STR() %then %do;
     		%rsk_terminate(key=irm_invalid_input_parameter, s1=IN_ENTITY_DS);
   	%end;
  		%if &CHILD_LIST eq %STR() %then %do;
     		%rsk_terminate(key=irm_invalid_input_parameter, s1=CHILD_LIST);
   	%end;
   		
   		
          /* Retrieve entities holding group's own positions */


          /* Retrieve child entities for the selected entity */
          %local i total_nm;
          %let i=1;
          %let total_nm=%rsk_attrn(&child_list, nobs);
          
          %do %while (&i. <= &total_nm.);
          
             data entity_info_&i;
                set &child_list(firstobs=&i. obs=&i.);
             run;
             
             /*In solo case, only retrieve non-legal entities*/
             proc sql noprint;
                create table _child_grp_&i
                as select * from &in_entity_ds
                where upcase(GROUP_ID) in (select ENTITY_ID from entity_info_&i.)
					%if %rsk_varexist(&IN_ENTITY_DS, SOLO_PARENT_FLG) and %upcase("&in_entity_role")="SOLO" %then %do;                
                  		and upcase(SOLO_PARENT_FLG)="Y"
                  	%end;
                ;
             quit;
             
             /*Remove any entries with ENTITY_ID = GROUP_ID*/
             data _child_grp_&i;
             	set _child_grp_&i;
             	if upcase(ENTITY_ID)=upcase(GROUP_ID) then delete;
             run;

                %let i=%eval(&i+1);
         %end;


         data child_entities;
            set
            %do i=1 %to &total_nm;
               _child_grp_&i
            %end;
            ;
         run;
         
         proc append base=&O_ENTITY_FINAL_DS data=child_entities;
         run;
            
         
         data &O_ENTITY_SUB_GRP_DS;
            set child_entities;
         run;

       
       proc datasets library=work nolist;
          delete _child_grp: entity_info: child_entities;
       quit;

%mend irm_get_child_parent_entity;
